//Similar to extractFile.ts, but instead of receiving and returning an ArrayBuffer, works with Node.js streams.

import * as stream from 'stream';
import * as zlib from 'zlib';
import { ISsnFileEntry } from '../interfaces/ISsnFileEntry';
import decryptStream from './streams/decryptStream';
import streamSetMaxLength from './streams/streamSetMaxLength';

/** Extracts the file with the given metadata from the stream.
 * The stream must already start at the .zip's local file header
 * and must transparently span across multiple disks if necessary.
 */
export default function extractFileStream(file: ISsnFileEntry, inputStream: stream.Readable): stream.Readable {
  const headerBuffer: Buffer = inputStream.read(30);
  const localFileHeader = new DataView(headerBuffer.buffer);

  //Local file header signature
  if (localFileHeader.getUint32(0, true) !== 0x04034B50) {
    throw new Error('Local file header had wrong magic');
  }
  //All fields in the local file header are copies of the central file header, so we can skip them.
  //FIXME: Maybe we should actually read these fields to verify that they are identical?
  //skip 22 bytes
  const localFilenameSize = localFileHeader.getUint16(26, true);
  const localExtraSize = localFileHeader.getUint16(28, true);

  //skip local file name and extra field
  inputStream.read(localFilenameSize + localExtraSize);

  //-------------------------------------------------

  let curStream = inputStream;

  //set max length
  const maxLength = streamSetMaxLength(curStream, file.compressedSize);
  curStream = maxLength;

  //pipe into decryption if file is encrypted
  if (file.decryptionKeys !== undefined) {
    const decryptTransform = decryptStream(curStream, file.decryptionKeys);
    curStream = decryptTransform;
  }

  //pipe into decompression
  const decompressTransform = zlib.createInflateRaw();
  curStream.pipe(decompressTransform);
  curStream = decompressTransform;

  //set max length
  const maxLength2 = streamSetMaxLength(curStream, file.size);
  curStream = maxLength2;

  return curStream;
}
